如何设置config.php
==========
版本：v1.0

本文介绍如何设置*config.php*文件。

本指引使用Markdown语法书写，若使用文本编辑器查看，请注意正文中的格式标记，这些标记并不是正文的一部分。

以下示例只适用于使用文本编辑器查看：

    *表示说明* （两侧各有一个星号）
    **表示强调**（两侧各有两个星号）
    `表示代码`（两侧各有一个反引号，中间的内容才是代码内容，输入代码时请不要包括反引号）
    <表示链接>（两侧各有一个尖括号）

更多关于Markdown语法的说明，请参见<http://wowubuntu.com/markdown/>。

如果您使用Windows，可以使用[MarkdownEditor](https://github.com/jijinggang/MarkdownEditor)来查看本文档。下载地址：<https://github.com/jijinggang/MarkdownEditor/blob/master/download/MarkdownEditor.zip?raw=true>

如果您使用Mozilla Firefox，亦可安装扩展[Markdown Viewer](https://addons.mozilla.org/zh-CN/firefox/addon/markdown-viewer/)，然后直接用Firefox打开本文件。

设置格式
----------
所有设置都使用常量声明来完成，常量声明的格式是：

    define('常量名', 常量值);

在*config.php*中，常量名即设置项的名称，由大写字母和下划线组成。  
常量值可以是字符串、布尔值、整型、浮点型等标量类型。修改设置时，请**仅修改常量值的部分**。

* 字符串由英文单引号（'）或双引号（"）包围，本文件中使用单引号；
* 布尔值只有`true`或`false`，不区分大小写，本文件中使用小写（注意有的设置项虽然是true或false，但其值为字符串而不是布尔值，需要用引号包围）；
* 整型数值直接用十进制表示；浮点型用小数点分隔的两段十进制数字来表示。

设置说明
----------
下面对每一个设置项进行说明，包括设置项的作用和取值。


### UI_LANG
字符串值  
默认值：`'en'`

设置界面的默认语言，用户可以在界面右上角的语言切换自行切换成其他语言。  
取值为合法的ISO 639-1语言缩写，目前仅支持`'en'`（英语）、`'zh-CN'`（中文（中国））和`'ja'`（日本语）。关于翻译事宜，请联系<qakcnyn@gmail.com>


### UI_THEME
字符串值  
默认值：`'default'`

设置主界面所使用的主题，该主题必须在*themes*目录下。


### SITE_TITLE
字符串值

设置网站的标题，显示在浏览器的标题栏中。


### SITE_DESCRIPTION
字符串值

设置网站的说明，包含在网站头部的`<meta>`元素中。


### SITE_KEYWORDS
字符串值

设置网站的关键字，包含在网站头部的`<meta>`元素中。


### ADMIN_EMAIL
字符串值

管理员的Email地址，会包含在页面的反馈信息中。


### SIZE_LIMIT
字符串值  
默认值：`'4M'`

表示限制上传文件大小的字节数，可以使用T、G、M、K作为后缀，不去分大小写。  
注意*php.ini*中的*post_max_size*和*upload_max_filesize*会影响到实际可上传的文件大小。


### UPLOAD_DIR
字符串值  
默认值：`'uploads'`

默认上传文件的目录，注意**目录权限要可以读写**。


### THUMB_DIR
字符串值  
默认值：`'thumbs'`

默认缩略图的目录，注意**目录权限要可以读写**。


### MANAGE_NAME
字符串值  
默认值： `'admin'`

管理界面的登录名。**请一定要修改**。


### MANAGE_PASSWORD
字符串值  
默认值：`'admin'`

管理界面的登陆密码。**请一定要修改**。


### CDN_ENABLED
布尔值  
默认值：`false`

是否开启CDN。如果您的使用CDN并且有多个域名均是主站的缓存或镜像，可以开启此选项。  
此选项只影响域名，无法影响到目录。即，您的图片保存在主站位置是*http://www.example.com/uploads/abs.jpg*，则启用CDN并添加*cdn1.example.com*，则会返回*http://cdn1.example.com/uploads/abs.jpg*作为图片地址，*uploads/abs.jpg*的部分会保持不变。  
启用CDN后域名会在如下设置的*CDN_LIST*中随机取用，如果仍想再返回的地址中包括主站地址，请将主站地址添加进*CDN_LIST*。


### CDN_LIST
字符串值  

设置CDN开启后随机取用域名的列表，多个域名以英文逗号（,）分隔，可以使用数字IP地址。


### CDN_HTTPS
字符串值  
默认值：`'both'`  
取值范围：`'forceon'`、`'forceoff'`、`'on'`、`'off'`、`'both'`

设置CDN是否开启HTTPS。

可以只设置一个值，代表全部开启或不开启。  
也可以设置由英文逗号（,）分隔的多个值，分别对应*CDN_LIST*的各个域名。如果多个值的数量少于*CDN_LIST*的域名数量，设置无效。如果某个域名想保持默认（*both*）可以留空，如`'both,,on'`分别设置第一个域名均可、第三个域名开启，第二个域名默认不开启。

各取值的取用情况如下：

* forceon：不论如何都HTTPS取用。
* forceoff：不论如何都以HTTP取用。
* on：只会在HTTPS访问上传页面的情况下以HTTPS取用。
* off：只会在HTTP访问上传页面的情况下以HTTP取用。
* both：在HTTP访问上传页面的情况下以HTTP取用，在HTTPS的情况下以HTTPS取用。

若上传页面以HTTP访问，会以HTTPS取用设置为*forceon*的域名，以HTTP取用*off*或*both*或*forceoff*的域名，不会取用*on*的域名。  
若上传页面以HTTPS访问，会以HTTPS取用设置为*on*或*both*或*forceon*的域名，以HTTP取用*forceoff*的域名，不会取用*off*的域名。

> #### 注意
> 建议不要依赖*forceon*或*forceoff*选项，而是用*on*、*off*、*both*来根据上传页面自适应协议。只有在全部强制启用或全部强制不启用的情况再使用*forceon*或*forceoff*。
> 
> 不要全部设置为*on*或*off*，这样在HTTP的情况下将无法取用到全部为*off*的域名，在HTPPS的情况下将无法取用到全部为*on*的域名，造成无法取到任何域名的问题。除非上传页面只能通过HTTP访问才能设置为全部*off*，或上传页面只能通过HTTPS访问才能全部设置为*on*。


### CDN_PORTS_HTTP
字符串值  
默认值：`''`  
取值范围：0～65535之间的整数（注意**此设置为字符串值**）

设置CDN域名的端口。留空表示使用默认值（HTTP为80）。
这个设置只对*CDN_HTTPS*设置为*forceoff*、*off*或*both*的域名有效。

可以设置一个值，表示全部使用某个端口。
也可以设置由英文逗号（,）分隔的多个值，分别对应*CDN_LIST*的各个域名。如果多个值的数量少于*CDN_LIST*的域名数量，设置无效。如果某个域名想保持默认（HTTP为80）可以留空，如`'667,,8080'`分别设置第一个域名端口为667、第三个域名端口为8080，第二个域名默认。


### CDN_PORTS_HTTPS
字符串值  
默认值：`''`  
取值范围：0～65535之间的整数（注意**此设置为字符串值**）

设置CDN域名的HTTPS端口，留空表示默认（HTTPS为443）。
这个设置只对*CDN_HTTPS*设置为*forceon*、*on*或*both*的域名有效。

可以设置一个值，表示全部使用某个端口。
也可以设置由英文逗号（,）分隔的多个值，分别对应*CDN_LIST*的各个域名。如果多个值的数量少于*CDN_LIST*的域名数量，设置无效。如果某个域名想保持默认（HTTPS为443）可以留空，如`'667,,8080'`分别设置第一个域名端口为667、第三个域名端口为8080，第二个域名默认。


### COPYRIGHT
字符串值  

这是在页面底部的版权声明，可以使用HTML代码。


### WATERMARK
布尔值  
默认值：`false`

是否启用水印。  
水印是永久加在图片上的，不可恢复。  
因为水印会导致GIF动画失效，因此只能对JPEG和PNG格式的图片打水印。

水印图片是*site-img/watermark.png*（严格区分大小写），可以使用半透明PNG图像。


### WATERMARK_MIN_SIZE
字符串值  
默认值：`'200x200'`

要加水印的图片的最小尺寸，小于这个尺寸的图片将不加水印。  
值由*宽x高*表示，均为正整数，只要图片的宽度小于设置的宽度，或者图片的高度小于设置的高度就不会加水印，而用两者都小于。


### WATERMARK_POS
字符串值  
默认值：`'10,10'`

要加水印的位置。  
值由*x,y*表示，均为整数，坐标是从图片的左上角开始计算的，x轴右延伸，y轴向下延伸。如果坐标为负数，则表示坐标为图片的宽减去x或高减去y的值。  
水印图片会以原始大小贴在坐标所表示的位置上，水印图片的左上角对准原图片坐标*(x,y)*的位置。  
如：假设图片大小为200x300，则*10,10*表示水印贴在图片坐标(10,10)的位置，*-10,10*表示(190,10)，*10,10*表示(10,290)，*-10,-10*表示(190,290)。